@@ -100,6 +100,23 @@ module xtb_setparam
    ! interface mode
    integer,parameter :: p_pcem_legacy = 1
    integer,parameter :: p_pcem_orca = 2
+   
+   type oniom_settings
+      integer  :: innerchrg
+         !! inner region charge
+      logical :: derived = .false.
+         !! set ONIOM optimization parameter g to derived value
+      logical :: cut_inner = .false.
+         !! to execute xtb just for checking inner region cut
+      logical :: fixed_chrgs= .false.
+         !! if charges for oniom explicitely given
+      logical :: silent = .false.
+         !! zo mute external output
+      logical :: logs = .false.
+         !! if optimization logs of inner regions are needed
+      integer:: ilog1, ilog2
+         !! log units
+  end type oniom_settings
 
    type qm_external
       character(len=:),allocatable :: path
@@ -107,6 +124,9 @@ module xtb_setparam
       character(len=:),allocatable :: input_file
       character(len=:),allocatable :: input_string
       logical :: exist
+         !! if input_file exist
+      logical :: oniom=.false.
+         !! special case of the oniom embedding 
    end type qm_external
 
    integer, parameter :: p_ext_vtb       = -1
@@ -118,6 +138,8 @@ module xtb_setparam
    integer, parameter :: p_ext_mopac     = 12
    integer, parameter :: p_ext_gfnff     = 13
    integer, parameter :: p_ext_oniom     = 14
+   integer, parameter :: p_ext_iff       = 15
+   integer, parameter :: p_ext_tblite    = 16
 
    integer, parameter :: p_run_scc    =   2
    integer, parameter :: p_run_grad   =   3
@@ -236,6 +258,7 @@ module xtb_setparam
    logical  :: shake_md = .true.
    logical  :: xhonly = .true.
    logical  :: honly = .false.
+   logical :: forcewrrestart = .false.
 
 !! ------------------------------------------------------------------------
 !  target rmsd value for bhess run in Ångström
@@ -307,8 +330,10 @@ module xtb_setparam
    character(len=:),allocatable  :: property_file
    logical  :: pr_esp = .false.
    character(len=:),allocatable  :: esp_gridfile
+   character(len=10) :: lmoinfo_fname='xtblmoinfo'
    logical  :: pr_molden_input = .false.
    logical  :: pr_lmo = .false.
+   logical  :: pr_local = .true.
    logical  :: pr_density = .false.
    logical  :: pr_spin_population = .false.
    logical  :: pr_spin_density = .false.
@@ -354,7 +379,7 @@ module xtb_setparam
    ! pcharge input file
    character(len=:),allocatable :: pcem_file
    character(len=:),allocatable :: pcem_grad
-   logical  :: pcem_orca   = .true.
+   logical  :: pcem_orca   = .false.
 !  controls which interactions included in the Fockian depend on the
 !  external point charges
    logical  :: pcem_l_es   = .true.
@@ -386,7 +411,13 @@ module xtb_setparam
    real(wp) :: ex_open ! set to 0.5/-0.5 in .xtbrc, respectively
 
 !! ------------------------------------------------------------------------
+!  ONIOM
+!! ------------------------------------------------------------------------
+   type(oniom_settings) :: oniom_settings
 
+!! ------------------------------------------------------------------------
+!  External settings
+!! ------------------------------------------------------------------------
    type(qm_external) :: ext_driver
    type(qm_external) :: ext_orca
    type(qm_external) :: ext_turbo
@@ -427,8 +458,11 @@ module xtb_setparam
 
 !  character(len=80) :: inputname = ''
    character(len= 4) :: pgroup = 'C1  '
-
+!! ------------------------------------------------------------------------
+    
    end type
+   
+
    type(TSet) :: set
 
    type(env_setvar) :: xenv
@@ -438,6 +472,7 @@ module xtb_setparam
 
 contains
 
+
 subroutine initrand
    implicit none
    integer :: i,j
