@@ -34,6 +34,7 @@ subroutine collect_gfn2(testsuite)
       new_unittest("gbsa", test_gfn2gbsa_api), &
       new_unittest("salt", test_gfn2salt_api), &
       new_unittest("pcem", test_gfn2_pcem_api), &
+      new_unittest("pcem_io", test_gfn2_pcem_io), &
       new_unittest("mindless-basic", test_gfn2_mindless_basic), &
       new_unittest("mindless-solvation", test_gfn2_mindless_solvation), &
       new_unittest("dmetal", test_gfn2_dmetal), &
@@ -518,6 +519,149 @@ subroutine test_gfn2_pcem_api(error)
 
 end subroutine test_gfn2_pcem_api
 
+subroutine test_gfn2_pcem_io(error)
+   use xtb_mctc_accuracy, only : wp
+   use xtb_mctc_io, only : stdout
+
+   use xtb_type_options
+   use xtb_type_molecule
+   use xtb_type_restart
+   use xtb_type_param
+   use xtb_type_data
+   use xtb_type_pcem
+   use xtb_type_environment
+   use xtb_embedding, only: read_pcem
+
+   use xtb_xtb_calculator, only : TxTBCalculator, newXTBCalculator, newWavefunction
+   use xtb_main_setup, only : addSolvationModel
+
+   type(error_type), allocatable, intent(out) :: error
+
+   real(wp),parameter :: thr = 1.0e-10_wp
+   integer, parameter :: nat = 12, nat2 = nat/2
+   integer, parameter :: at(nat) = [8,1,1, 8,1,1, 8,1,1, 8,1,1]
+   real(wp),parameter :: xyz(3,nat) = reshape([&
+      &-2.75237178376284_wp, 2.43247309226225_wp,-0.01392519847964_wp, &
+      &-0.93157260886974_wp, 2.79621404458590_wp,-0.01863384029005_wp, &
+      &-3.43820531288547_wp, 3.30583608421060_wp, 1.42134539425148_wp, &
+      &-2.43247309226225_wp,-2.75237178376284_wp, 0.01392519847964_wp, &
+      &-2.79621404458590_wp,-0.93157260886974_wp, 0.01863384029005_wp, &
+      &-3.30583608421060_wp,-3.43820531288547_wp,-1.42134539425148_wp, &
+      & 2.75237178376284_wp,-2.43247309226225_wp,-0.01392519847964_wp, &
+      & 0.93157260886974_wp,-2.79621404458590_wp,-0.01863384029005_wp, &
+      & 3.43820531288547_wp,-3.30583608421060_wp, 1.42134539425148_wp, &
+      & 2.43247309226225_wp, 2.75237178376284_wp, 0.01392519847964_wp, &
+      & 2.79621404458590_wp, 0.93157260886974_wp, 0.01863384029005_wp, &
+      & 3.30583608421060_wp, 3.43820531288547_wp,-1.42134539425148_wp], shape(xyz))
+   type(scc_options),parameter :: opt = scc_options( &
+      &  prlevel = 2, maxiter = 30, acc = 1.0_wp, etemp = 300.0_wp, grad = .true. )
+
+   type(TMolecule)    :: mol
+   type(TRestart) :: chk
+   type(TEnvironment) :: env
+   type(scc_results) :: res
+   type(TxTBCalculator) :: calc
+
+   real(wp) :: energy, sigma(3,3)
+   real(wp) :: hl_gap
+   real(wp),allocatable :: gradient(:,:)
+
+   integer :: tmp_unit
+
+   ! setup the environment variables
+   call init(env)
+
+   call init(mol, at, xyz)
+
+   allocate(gradient(3,mol%n))
+   energy = 0.0_wp
+   gradient = 0.0_wp
+
+   call newXTBCalculator(env, mol, calc, method=2)
+   call newWavefunction(env, mol, calc, chk)
+
+   call calc%singlepoint(env, mol, chk, 2, .false., energy, gradient, sigma, &
+      & hl_gap, res)
+
+   call check_(error, hl_gap, 12.391144584178_wp, thr=thr)
+   call check_(error, energy,-20.323978512117_wp, thr=thr)
+   call check_(error, norm2(gradient),0.78119239557115E-02_wp, thr=thr)
+
+   call check_(error, gradient(1,5),-0.22192122053513E-02_wp, thr=thr)
+   call check_(error, gradient(2,2), 0.22192122053512E-02_wp, thr=thr)
+   call check_(error, gradient(1,4), 0.95621597761913E-03_wp, thr=thr)
+   call check_(error, gradient(3,6),-0.11904153838296E-02_wp, thr=thr)
+
+   ! reset
+   call mol%deallocate
+   deallocate(gradient)
+
+   call init(mol, at(:nat2), xyz(:, :nat2))
+   allocate(gradient(3,mol%n))
+   energy = 0.0_wp
+   gradient = 0.0_wp
+
+   call newXTBCalculator(env, mol, calc, method=2)
+
+   open(newunit=tmp_unit, Status="Scratch")
+      write(tmp_unit,'(a)') &
+      "6",&
+      "-0.69645733    2.75237178376284    -2.43247309226225    -0.01392519847964  O", &
+      " 0.36031084    0.93157260886974    -2.79621404458590    -0.01863384029005  H", &
+      " 0.33614649    3.43820531288547    -3.30583608421060     1.42134539425148  H", &
+      "-0.69645733    2.43247309226225     2.75237178376284     0.01392519847964  O", &
+      " 0.36031084    2.79621404458590     0.93157260886974     0.01863384029005  H", &
+      " 0.33614649    3.30583608421060     3.43820531288547    -1.42134539425148  H"
+   Call read_pcem(tmp_unit,env,calc%pcem,calc%xtbData%coulomb)
+   close(tmp_unit,Status="Delete")
+
+   calc%pcem%grd = 0.0_wp
+
+   call newWavefunction(env, mol, calc, chk)
+   call calc%singlepoint(env, mol, chk, 2, .false., energy, gradient, sigma, &
+      & hl_gap, res)
+
+   call check_(error, hl_gap, 12.718203165741_wp, thr=thr)
+   call check_(error, energy,-10.160927754235_wp, thr=thr)
+   call check_(error, norm2(gradient),0.21549557655285E-01_wp, thr=thr)
+
+   call check_(error, gradient(1,5),-0.20326749264006E-02_wp, thr=thr)
+   call check_(error, gradient(2,2), 0.33125459724368E-02_wp, thr=thr)
+   call check_(error, gradient(1,4),-0.11929405659085E-02_wp, thr=thr)
+   call check_(error, gradient(3,6),-0.16607682747438E-02_wp, thr=thr)
+
+   call check_(error, norm2(calc%pcem%grd),0.10043976337709E-01_wp, thr=thr)
+   call check_(error, calc%pcem%grd(1,5),-0.24831958012524E-03_wp, thr=thr)
+   call check_(error, calc%pcem%grd(2,2), 0.14208444722973E-02_wp, thr=thr)
+   call check_(error, calc%pcem%grd(1,4), 0.37466852704082E-02_wp, thr=thr)
+   call check_(error, calc%pcem%grd(3,6), 0.65161344334732E-03_wp, thr=thr)
+
+   ! reset
+   energy = 0.0_wp
+   gradient = 0.0_wp
+   calc%pcem%grd = 0.0_wp
+   calc%pcem%gam = 999.0_wp ! point charges
+
+   call newWavefunction(env, mol, calc, chk)
+   call calc%singlepoint(env, mol, chk, 2, .false., energy, gradient, sigma, &
+      & hl_gap, res)
+
+   call check_(error, hl_gap, 13.024345612330_wp, thr=thr)
+   call check_(error, energy,-10.168788268962_wp, thr=thr)
+   call check_(error, norm2(gradient),0.18113624400926E-01_wp, thr=thr)
+
+   call check_(error, gradient(1,5),-0.50646289499094E-03_wp, thr=thr)
+   call check_(error, gradient(2,2), 0.25468320656932E-02_wp, thr=thr)
+   call check_(error, gradient(1,4),-0.74927880093291E-02_wp, thr=thr)
+   call check_(error, gradient(3,6),-0.13248514654811E-02_wp, thr=thr)
+
+   call check_(error, norm2(calc%pcem%grd),0.18721791896294E-01_wp, thr=thr)
+   call check_(error, calc%pcem%grd(1,5),-0.21573703225712E-02_wp, thr=thr)
+   call check_(error, calc%pcem%grd(2,2), 0.25653662154150E-02_wp, thr=thr)
+   call check_(error, calc%pcem%grd(1,4), 0.12124342986218E-01_wp, thr=thr)
+   call check_(error, calc%pcem%grd(3,6), 0.12140575062433E-02_wp, thr=thr)
+
+end subroutine test_gfn2_pcem_io
 
 subroutine test_gfn2_mindless_basic(error)
    use xtb_mctc_accuracy, only : wp
@@ -793,23 +937,23 @@ subroutine test_gfn2_mindless_cosmo(error)
       & "h2o", "chcl3", "thf", "acetonitrile", "toluene", &
       & "ch2cl2", "ether", "methanol", "cs2", "dmso"]
    real(wp), parameter :: ref_energies(10) = &
-      &[-30.376518115875_wp,-24.102578449616_wp,-23.754139493763_wp, &
-      & -22.766727017808_wp,-27.757889554228_wp,-18.587445483567_wp, &
-      & -33.445663939387_wp,-30.003461899598_wp,-20.573488191967_wp, &
-      & -25.670456523922_wp]
+      &[-30.398257570644_wp,-24.125868305041_wp,-23.787177902522_wp, &
+      & -22.779539008049_wp,-27.783054701847_wp,-18.610486346192_wp, &
+      & -33.456896298026_wp,-30.008556230192_wp,-20.588957648989_wp, &
+      & -25.684670492104_wp]
    real(wp), parameter :: ref_gnorms(10) = &
-      &[0.065115699688_wp, 0.057742250527_wp, 0.041885898146_wp, &
-      & 0.072134973561_wp, 0.048230827198_wp, 0.052250268149_wp, &
-      & 0.045597725657_wp, 0.062065139853_wp, 0.054691676440_wp, &
-      & 0.045829509846_wp]
+      &[0.070117675966_wp, 0.060272066809_wp, 0.048063628868_wp, &
+      & 0.076296084561_wp, 0.050041916284_wp, 0.051542825738_wp, &
+      & 0.046066119467_wp, 0.060436009320_wp, 0.058023381592_wp, &
+      & 0.050440402702_wp]
    real(wp), parameter :: ref_hlgaps(10) = &
-      &[2.608938030439_wp, 1.390465293950_wp, 0.169294532274_wp, &
-      & 1.254915268362_wp, 2.137937542332_wp, 2.267712835326_wp, &
-      & 2.669493177775_wp, 0.840230142925_wp, 3.341412667263_wp, &
-      & 0.393537743933_wp]
+      &[3.173028297842_wp, 1.553006040621_wp, 0.324761161079_wp, &
+      & 1.233091187215_wp, 1.918282889245_wp, 2.067664999523_wp, &
+      & 2.757505697757_wp, 0.846361565116_wp, 3.336544801399_wp, &
+      & 0.315118656281_wp]
 
    call init(env)
-   do iMol = 1, 2
+   do iMol = 1, 10
 
       call getMolecule(mol, mindless(iMol))
 
